use memmap2::Mmap;
use crate::app_error::AppError;
use crate::{pe_parse::pe::ReadOnlyPE, services::file};

/// 应用的状态结构体
#[derive(Default)]
pub struct AppState {
    /// 文件路径
    pub file_path: Option<String>,
    /// 文件Mmap的只读内存映射
    mmap: Option<Mmap>,
    /// 文件是否是64位
    pub is_64_bit: Option<bool>
}

impl AppState {
    /// 初始化打开文件
    pub fn init_open_file(&mut self, file_path: &str) -> Result<(), AppError> {
        self.file_path = Some(file_path.to_string());
        self.mmap = Some(file::mmap_file(file_path)?);
        // 读取PE格式来判断是否是64位
        let mmap: &Mmap = self.mmap.as_ref().unwrap();
        self.is_64_bit = Some(mmap.is_64_bit()?);
        Ok(())
    }

    /// 获取文件内存映射的引用
    pub fn get_mmap_ref(&self) -> Result<&Mmap, AppError> {
        self.mmap.as_ref().ok_or(AppError::NoFileOpened)
    }

    /// 设置文件内存映射
    pub fn set_mmap(&mut self, mmap: Mmap) {
        self.mmap = Some(mmap);
    }

    /// 在指定偏移处写入数据，此举会同步修改文件内容
    pub fn write_data(&mut self, offset: usize, data: &[u8]) -> Result<(), AppError> {
        // 1. 重新以可读可写打开文件
        let file_path = self.file_path.as_ref().ok_or(AppError::NoFileOpened)?;
        let mut rw_mmap = file::mmap_file_rw(file_path)?;
        // 2. 向内存映射中写入数据
        rw_mmap[offset..(offset + data.len())].copy_from_slice(data);
        rw_mmap.flush()?;

        // 3. 重新打开文件
        self.mmap = Some(file::mmap_file(file_path)?);
        self.is_64_bit = Some(self.get_mmap_ref()?.is_64_bit()?);

        Ok(())
    }

}